/****************************************************************************
*
* (C) 2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module         : HELPER.C
* Function       : Some useful helper functions
* ---------------------------------------------------------------------------

$Header: HELPER.C, 7, 13.04.2004 11:34:09, Andre Pribil$

*****************************************************************************/

#include "helper.h"
#include <stdarg.h>

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 Helper API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces HELPER\.H><efont> |

 @libcat1x  | Character output | helper01 |
    helper_strout | String output |
 @libcat1x  | Character output | helper02 |
    helper_chrout | Char output |
 @libcat1x  | Character output | helper03 |
    helper_dezout | Dezimal output |
 @libcat1x  | Character output | helper04 |
    helper_hexdig | Hex digit output |
 @libcat1x  | Character output | helper05 |
    helper_hexout | Hex value output |
 @libcat1x  | Character output | helper06 |
    helper_printf | Printf output |

 @libcat2x  |  Miscellaneous  | helper07 |
    helper_alloc_rtos_mem | Allocate RTOS memory |
 @libcat2x  |  Miscellaneous  | helper08 |
    helper_release_rtos_mem | Release allocated RTOS memory |
*/


#ifdef _helper01_
/*
 @doc DOC_helper01_
 ---------------------------------------------------------------
 Strout
 ---------------------------------------------------------------
 @clib  helper_strout | String output function |
        <cfont>void helper_strout ( const char far *str ); <efont> |
 @parm  char far | str  | Zero terminated output string to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x09<linkmid>documentation<linktail>
        for more details.
*/
void helper_strout( const char far *s )
{
  asm push ds
  asm lds  dx, s
  asm mov  ah, 0x9
  asm int  0x21
  asm pop  ds
} // helper_strout
#endif


#ifdef _helper02_
/*
 @doc DOC_helper02_
 ---------------------------------------------------------------
 Chrout
 ---------------------------------------------------------------
 @clib  helper_chrout | Character output function |
        <cfont>int helper_chrout ( char ch, unsigned int zero ); <efont> |
 @parm  char  | ch    | Character to print over STDOUT
 @parm  int   | zero  | Zero-Mode<nl>
                        if ch>=10 zero is ignored.<nl>
                        if ch\<10 zero is ignored but the character printed out is ch + 48.<nl>
                        if ch==0 and zero==0 character print is ch + 48.<nl>
                        if ch==0 and zero \< 0 no character is printed<nl>
                        if ch==0 and zero > 0 a space character is printed.<nl>
 @rdesc Value in zero is returned
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
*/
int helper_chrout( char ch, register zero )
{
  if( ch < 10 )
  {
    if( ch || ( zero == 0 ))
    {
      zero = 0;
      _AL = '0' + ch;

    }
    else
    {
      if( zero < 0 )
        return zero;

      _AL = ' ';

    } // if

  }
  else
  {
    _AL = ch;
  }

  _AH = 0xE;
  _BH = 0;
  asm int 10h

  return zero;

} // helper_chrout
#endif


#ifdef _helper03_
/*
 @doc DOC_helper03_
 ---------------------------------------------------------------
 Dezout
 ---------------------------------------------------------------
 @clib  helper_dezout | Character output function |
        <cfont>void helper_dezout ( unsigned int x, unsigned int zero ); <efont> |
 @parm  int   | x     | Decimal value to print over STDOUT
 @parm  int   | zero  | Zero-Mode<nl>
                        if zero==0, leading zeros are printed as zeros.<nl>
                        if zero\<0, leading zeros are not printed<nl>
                        if zero>0, leading zeros are printed as spaces.<nl>
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
*/
void helper_dezout( register x, register zero )
{
  zero = helper_chrout( x / 1000,zero );
  x %= 1000;

  zero = helper_chrout( x / 100,zero );
  x %= 100;

  helper_chrout( x / 10,zero );
  helper_chrout( x % 10,0 );

} // helper_dezout
#endif


#ifdef _helper04_
/*
 @doc DOC_helper04_
 ---------------------------------------------------------------
 Hexdig
 ---------------------------------------------------------------
 @clib  helper_hexdig | Hexadecimal digit output function |
        <cfont>void helper_hexdig ( unsigned char dig ); <efont> |
 @parm  char  | dig  | Hexadecimal digit (nibble) to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
*/
void helper_hexdig( unsigned char dig )
{
  dig &= 0xF;

  if( dig > 9 )
    dig += 'A' - 10;
  else
    dig += '0';

  helper_chrout( dig,-1 );

} // helper_hexdig
#endif


#ifdef _helper05_
/*
 @doc DOC_helper05_
 ---------------------------------------------------------------
 Hexout
 ---------------------------------------------------------------
 @clib  helper_hexout | Hexadecimal integer output function |
        <cfont>void helper_hexout ( unsigned int hex ); <efont> |
 @parm  int  | hex  | Hexadecimal integer to print over STDOUT
 @rdesc -- none --
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
*/
void helper_hexout( unsigned hex )
{
  helper_chrout( '0', -1 );
  helper_chrout( 'x', -1 );
  helper_hexdig( hex >> 12 );
  helper_hexdig( hex >>  8 );
  helper_hexdig( hex >>  4 );
  helper_hexdig( hex       );

} // helper_hexout
#endif

#ifdef _helper06_
/*
 @doc DOC_helper06_
 ---------------------------------------------------------------
 printf
 ---------------------------------------------------------------
 @clib  helper_printf | Output function like original printf (but consumes less memory) |
        <cfont>int helper_printf ( const char _FAR *fmt, ... ); <efont> |
 @parm  char *  | fmt  | Format string, max. 180 characters
 @rdesc If 0, function is not supported on current RTOS.
 @devnote Pointers which are passed to this function must be of type "far".<nl>
          e.g. <cfont>helper_printf("%s", (char far *)string);<efont>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x56<linkmid>documentation<linktail>
        for more details.
*/
int helper_printf(const char _FAR *fmt, ...)
{
  static int firsttime=1;
  static     Sprintf_Fptr sprintf_fptr;

  char       bfr[180];
  va_list    parg;
  int        rslt;

  if(firsttime)
  {
     //get pointer of rtos internal sprintf function
     asm{
          mov ah,0x56
          int 0xA0
     }
     if (_AX==0)
     {
       sprintf_fptr =  MK_FP(_ES,_DI);
       firsttime=0;
     }
     else
     {
       return 0;
     }
  }
  va_start(parg,fmt);
  rslt=(*sprintf_fptr)(bfr,fmt,parg);
  va_end(parg);
  helper_strout(bfr);
  return rslt;
} // helper_printf
#endif

#ifdef _helper07_
/*
 @doc DOC_helper07_
 ---------------------------------------------------------------
 Miscellaneous
 ---------------------------------------------------------------
 @clib  helper_alloc_rtos_mem | Allocate memory from @CHIP-RTOS memory pool (Int21h function 48h) |
        <cfont>unsigned int helper_alloc_rtos_mem(unsigned long size);<efont> |
 @parm  long  | size  | Requested memory size in bytes
 @rdesc If 0, memory allocation failed.
        Else segment of the allocated memory area

 @comm The returned segment can be used to build a normalized pointer to the allocated memory area e.g.
       <cfont>seg=helper_alloc(1024);<nl>
       ptr = MK_FP(seg,0);<efont>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x48<linkmid>documentation<linktail>
        for more details.
*/
unsigned int helper_alloc_rtos_mem(unsigned long size)
{
  _BX = (unsigned int)((size + 15) >> 4);     //Required paragraph count-> BX.
  _AH = 0x48;
  asm int 21h
  return _AX;
}
#endif

#ifdef _helper08_
/*
 @doc DOC_helper08_
 ---------------------------------------------------------------
 Miscellaneous
 ---------------------------------------------------------------
 @clib  helper_release_rtos_mem | Release allocated RTOS memory (Int21h function 49h) |
        <cfont>int helper_release_rtos_mem(unsigned int seg);<efont> |
 @parm  int  | seg  | Segment of the memory area to be released
 @rdesc 0 success<nl>
        -1 failed<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../dosemu.htm#0x210x49<linkmid>documentation<linktail>
        for more details.
*/
int helper_release_rtos_mem(unsigned int seg)
{
  unsigned int ax_tmp;

  _ES=seg;
  _AH=0x49;
  asm int 21h

  ax_tmp=_AX;

  if((_FLAGS & 0x0001/*CF*/) && (ax_tmp==9))
  {
     return -1;
  }
  return 0;
}
#endif


